#!/usr/bin/perl -T
# $Id: ajax.cgi,v 1.5 2017/01/12 12:13:45 sofc Exp $
use strict;
use warnings;
use CGI qw(:standard :netscape);
use Elchemea;
use ElchemeaHeader;
use ElchemeaUser;
use ElchemeaCGI qw(:DEFAULT :html :cgi);

print header();
my $u = param('user');
my $p = param('pass');
unless (&ElchemeaCGI::login_ok($u,$p))
{
    print "Access denied!\n";
    exit;
}
my $user = ElchemeaUser->new($u);
my $session = $user->session;
my $device = param('device');
my $action = param('action');
unless (defined($device) && $device && defined($action) && $action)
{
    print "Undefined device or action found!\n";
    $action = '' unless ($action);
    print "$action\n\n";
    exit;
}

my $warn = '';
my $pot = Elchemea::new_PSTAT(Elchemea::get_config_value('equipment','potentiostat'),Elchemea::get_config_value('equipment','potentiostat_address'));
my $fra = Elchemea::new_FRA(Elchemea::get_config_value('equipment','FRA'),Elchemea::get_config_value('equipment','FRA_address'));

my $dev = undef; 
$dev = $pot if ($device eq 'POT');
$dev = $fra if ($device eq 'FRA');

unless ($dev)
{
    print "Undefined device ('$device' not FRA or POT)\n";
    exit;
}

if ($action eq 'measure_pot')
{
    unless ($pot)
    {
	print "No potentiostat found!\n";
	exit;
    }    
    my $str = $pot->run_dvm();
    if ($str =~ /\d/)
    {
	my ($U,$I) = split(/\s+/,$str);
	### Note this is only true for default setup of the potentiostat with working electrode on negative terminal! ###
	foreach ($U,$I)
	{
	    $_ = -$_;
	}
	print "U: $U V, I: $I A\n";
    }
    exit;
}


if ($action eq 'isalive')
{
    unless ($pot)
    {
	print "No potentiostat found!\n";
	exit;
    }  
    if ($pot->isallive())
    {
	print "OK";
    }
    else
    {
	print "Potentiostat not responding!";
    }
    exit;
}


print "Unknown action found: '$action'\n";
exit;
