#!/usr/bin/perl -T
# $Id: misc.cgi,v 1.4 2016/03/03 12:52:42 sofc Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Karin Soeren Koch           ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2005              ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use ElchemeaHeader;
use ElchemeaUser;
use Elchemea;
use Eurotherm;
use ElchemeaCGI qw(:DEFAULT :html :cgi);
use CGI qw(:standard :netscape);

if (param('ajax'))
{
    print header();
    unless (login_ok())
    {
        print "";
        exit;
    }
    my $user = ElchemeaUser->new(param('user'));
    my $a = param('action');
    if ($a eq 'test')
    {
	my $type = get_CGI_value('value');
	if ($type eq 'test_temperature')
	{
	    my $tty = $user->get_config_value('temperature','tty');
	    $tty = 'ttyS0' unless ($tty);
	    my $channel = $user->get_config_value('temperature','address');
	    my $comm = $user->get_config_value('temperature','controler_mode');
	    my $cont = $user->get_config_value('temperature','controler_type');
	    my $mode = $user->get_config_value('temperature','type');
	    my $controler = new_controler($cont,$tty,$channel,$comm,$mode);
	    unless ($controler)
	    {   
		print "Error: Could not initialise controler '$cont' at tty $tty and address $channel";
		exit;
	    }
	    my $temp = $controler->get_active_temp();
	    my $sp = $controler->get_temp();
	    my $ramp = $controler->get_ramp();
	    my @list = ('temperature','ramprate','active temperature');
	    my $c = 0;
	    foreach ($temp,$ramp,$sp)
	    {
		print "Warning: could not determine $list[$c]!\n" unless (defined($_));
		$c++;
	    }
      	    print "Temperature is '$temp' C, setpoint is '$sp' C with '$ramp' C/hour";
	    exit;
	}
	if ($type eq 'test_datalog')
	{
	    eval {
		print "testing datalog\n";
		print $user->get_channels();
	    };
	    if ($@)
	    {
		print $@;
	    }
	    exit;
	}
    }
    exit;
}



my $user;
my $bc_admin = undef;
if (!defined(param))
{
    print_header("Please log in to the Elchemea");
    print '<center>';
    &ElchemeaCGI::login;
    print '</center>';
    &print_end;
}
else
{
    if (login_ok())
    { 
	$user = ElchemeaUser->new(param('user'));
	&ElchemeaCGI::print_header("Elchemea miscellaneous setup",-onload=>'load_all()');   
	top_nav_bar_start();
	param(-name=>'action',-value=>'') unless (param('action'));
	$_ = param('action');
      CASE:
	{
	    ### Insert case statements ###
	    ### Default response ####
	    &main_page;
	}
	print_end();
    }
    else 
    {
	not_auth();
    }
}
    
exit;

sub main_page
{
    top_nav_bar_button('config.cgi','','','Manually edit configuration','s6');
    top_no_button();
    top_no_button();
    tab_newrow();
    js_back();
    top_js_return('#006600','white');
    top_no_button();
    top_nav_bar_button('keithley.cgi','','','Setup datalogging channels','s3');
    top_no_button();
    top_nav_bar_button('main.cgi','','','To menu','s4');
    end_top_bar();
    my $pot = Elchemea::new_PSTAT(Elchemea::get_config_value('equipment','potentiostat'),Elchemea::get_config_value('equipment','potentiostat_address'));
    my $fra = Elchemea::new_FRA(Elchemea::get_config_value('equipment','FRA'),Elchemea::get_config_value('equipment','FRA_address'));
    print '<center>';
    print '<table cellspacing=5 cellpadding=5 class=tabel"><tr>';
    print '<td width="5%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="17%"></td>';
    print '<td width="10%"></td>';
    print '<td width="3%"></td>';
    print '<td width="5%"></td>';
    print '</tr>';
    my @tags = ();
    my @yn = ('','Yes','No');
    my @mt = ('None','At start','At end','At start and at end');
    foreach my $sec (get_config_value('misc','sections'))
    {
	print '<tr><td colspan=11 class="heading"> '.ucfirst(lc($sec)).' setup </td></tr>';
	my @cf = get_config_value('misc','keys_'.$sec);
	my %setup = ();
	foreach my $tag (@cf)
	{
	    $setup{$tag} = $user->get_config_value($sec,$tag);
	}
	push @cf,'test_'.$sec if ($sec =~ /temp|datalog/i);
	my $count = 0;
	foreach my $key (@cf)
	{
	    print '<tr><td></td>' unless ($count);
	    print '<th>';
	    my $tag = $key;
	    $tag =~ s/_/ /g;
	    if ($key =~ /test_\w+/)
	    {
		print button(-name=>ucfirst($tag),-class=>'s3',-onclick=>"test('$key')");
	    }
	    else
	    {
		print $tag;
		print '</th><td>';
		my @l = ();
		@l = @yn if ($key =~ /logfile/);
		@l = @mt if ($key =~ /measure/);
		@l = Eurotherm::get_list() if ($key eq 'controler_type');
		@l = ('integer','decimal') if ($key eq 'type');
		@l = ('bisynch','modbus') if ($key eq 'controler_mode');
		if (@l>1)
		{
		    print popup_menu(-name=>$key,-values=>[(@l)],-class=>'textfield',-default=>$setup{$key},-force=>1,-onchange=>"change_val(this,'$sec')",-id=>$key);
		}
		else
		{
		    print textfield(-name=>$key,-value=>$setup{$key},-class=>'textfield',-force=>1,-onchange=>"change_val(this,'$sec')",-id=>$key);
		}
		push @tags,$key;
		print '</td><th id="tag_'.$key.'">';
		$_ = $tag;
	      CASE:
		{
		    /resis/i and do {print 'Ohm'};
		    /cur/i and do {print 'A'};
		    /delay|time/ and do {print 's'};
		}
	    }
	    print '</th>';
	    if ($count++ > 1)
	    {
		$count = 0;
		print '</tr>',"\n";
	    }
	}
    }
    print '</table>';
    print '</center>';

    my $valuearr = "var vals = [];\n";
    for (my $i = 0; $i < @tags; $i++ )
    {
        $valuearr .= 'vals['.$i.'] = "'.$tags[$i].'";'."\n";
    }

    print <<END;


<script language="javascript">

$valuearr

function load_all()
{

}

function set_val(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	var obj = document.getElementById(arr[1]);
	obj.disabled = false;
    }
    else
    {
	alert(arr[0]);
    }
}

function change_val(element,s)
{
    var n = element.name;
    var val = element.value;
    element.disabled = true;
    run_ajax('change_config.cgi','&action=set_value&section='+s+'&key='+n+'&value='+val,set_val);
}

function test(type)
{
    run_ajax('misc.cgi','&action=test&value='+type,alert);
}



</script>

END

}
